/*
  Uno1284P & Kanji LCD Shield - Text Demo #1

  本スケッチは "Uno1284P 基板" 及び "漢字 LCD シールド" で、デモ表示するスケッチ例です。
  コンパイルすると 126kB 強になるので、Uno1284P 基板に換装した Arduino でなければ動作しません。

  Circuit:
    pins A4(SDA), A5(SCL) for TWI communication in Arduino standard.
    pins SDA, SCL for TWI communication in Arduino UNO R3, Leonardo.

  created 2/feb/2013 by I.Kato @ BS21 Lab

  This code is in the public domain.
*/

// TWI 通信用ライブラリ：必須です
#include <Wire.h>

// 漢字 LCD 用ライブラリ
#include <KanjiLCD.h>


// 漢字 LCD シールドの TWI アドレスを 0x38 以外に設定した場合は、以下の値を書きかえてください。
// TWI 通信できていない時は、漢字 LCD シールド自身に現在設定されている TWI(i2c) アドレスが
// 表示されているはずなので、御確認ください。
#define KLCD_TWI_ADDR  0x38


// 漢字 LCD クラスのインスタンス生成
KanjiLCD klcd (KLCD_TWI_ADDR); // 漢字 LCD シールドの TWI アドレスを指定する


size_t msg_count; // 表示させるメッセージ文字列の番号

uint8_t right_sw; // 漢字 LCD シールドのスイッチの状態


// デモ表示用メッセージ文字列の配列
// ※巨大なので、プログラムメモリに格納します。PROGMEM/HIGH_PROGMEM 指定が必須です。

PROGMEM const char demo_msg1 [32500] = { // PROGMEM 指定なので、フラッシュメモリの前半 64kB に格納されます。
  "♪漢字LCDｼｰﾙﾄﾞ♫" "全角８文字×２行", // オープニング・メッセージ
};
PROGMEM const char demo_msg2 [32500] = { // PROGMEM 指定なので、フラッシュメモリの前半 64kB に格納されます。
  "組立ｷｯﾄ･要半田付" "✓価格￥4,780-也", // ２番目のメッセージ
};
HIGH_PROGMEM const char demo_msg3 [30000] = { // HIGH_PROGMEM 指定なので、フラッシュメモリの後半 64kB に格納されます。
  "☆JIS 第一水準字" "utf8対応,i2c通信", // ３番目のメッセージ
};
HIGH_PROGMEM const char demo_msg4 [30000] = { // HIGH_PROGMEM 指定なので、フラッシュメモリの後半 64kB に格納されます。
  "半角 157字、及び" "全角3737字に対応", // ４番目のメッセージ
};

// デモ用メッセージ文字列の配列の個数
size_t msg_size = 4;


char buff [10000]; // SRAM 上に 10000バイトを確保


// 初期化関数
void setup()
{
  UseHighProgMem(); // HIGH_PROGMEM 使用時は必須です。

  // 必ず、最初に Wire ライブラリを初期化させておいてください。
  Wire.begin();

  // 続いて、漢字 LCD ライブラリを初期化します。
  // 符号化方式は UTF-8 方式がデフォルトなので、特に指定しなくても OK です。
  // 初期化に成功するとは、画面はクリアされ、カーソル位置は (0,0) になります。
  // 漢字 LCD シールドは、電源投入直後は自動的に Version と TWI アドレスを表示していますので、
  // 初期化に失敗した場合は、TWI アドレスが KLCD_TWI_ADDR と一致しているか確認してください。
  klcd.begin(16, 2);

  // オープニングのメッセージ文字列を表示させます。
  // プログラムメモリ上の文字列を表示させる時は、KanjiLCD::print_P 関数を呼び出して下さい。
  klcd.print_P(demo_msg1); // demo_msg1 配列は PROGMEM 指定なので、xxx_P 関数が使用できる（HIGH_PROGMEM 指定の配列だとダメ）

  msg_count = 0;

  // スイッチは押されていない状態にする
  right_sw = false;
}

// ループ関数
void loop()
{
  // 最初に漢字 LCD シールドの、現在のステータスを取得します。
  // これを呼び出しておかないと、以下の getRightSW 関数が、何時までも更新されない事になってしまいます。
  // 漢字 LCD シールドがコマンド処理中で忙しい時は、getStatus 関数はステータス取得に失敗して、KLCD_ERR_BUSY が返って来ます。
  // なので、ステータス取得成功を意味する KLCD_ERR_NONE が返ってきた時だけ、処理を進めるようにして下さい。
  if (klcd.getStatus() == KLCD_ERR_NONE) {
    // ステータス取得成功した

    // 右スイッチの処理
    if (klcd.getRightSW()) { // 右スイッチが押されている
      // 押され続けている間はフラグを立てるだけの処理
      right_sw = true;

    } else { // 右スイッチが離された
      if (right_sw == true) { // さっきまで右スイッチが押されていた

        // 次のメッセージ文字列を表示させる
        msg_count++; // 1 加算
        if (msg_count >= msg_size) { // メッセージ配列の末尾に達した
          msg_count = 0; // 最初のメッセージ番号に戻る
        }

        // 画面をクリアし、カーソル位置を (0,0) にします。約60ms かかります。
        klcd.clear();

        // 該当するメッセージ文字列を表示させます
        const char *src = NULL;
        char *dst = buff;
        switch (msg_count) {

        case 0: // PROGMEM 指定の場合
          src = demo_msg1;
          while (*dst++ = pgm_read_byte(src++)) { // '\0' までループ・コピーする
            ;
          }
          break;

        case 1: // PROGMEM 指定の場合
          memcpy_P (buff, demo_msg2, sizeof(buff)); // memcpy_P 関数を使ってコピーする
          break;

        case 2: // HIGH_PROGMEM 指定の場合
          src = demo_msg3;
          while (*dst++ = pgm_read_byte_hi(src++)) { // '\0' までループ・コピーする
            ;
          }
          break;

        case 3: // HIGH_PROGMEM 指定の場合
          memcpy_PH (buff, demo_msg4, sizeof(buff)); // memcpy_PH 関数を使ってコピーする
          break;
        }

        klcd.print(buff); // LCD 画面に表示する

        // 右スイッチ・フラグをクリア
        right_sw = false;
      }
    }
  }

  // 10ms 待ちを入れる
  // 漢字 LCD シールドのスイッチのチェック間隔が、約 10ms (変更不可) なので、
  // これよりも短い間隔で getStatus 関数を呼び出しても、前と同じスイッチ状態が読み出されるだけです。
  // なので、10ms 以上の間をおいてから、getStatus 関数を呼び出すようにしてください。
  delay(10);
}
